@@ -20,7 +20,7 @@
 #include "qgis_core.h"
 #include "qgis_sip.h"
 #include <QObject>
-#include <QMutex>
+#include <QRecursiveMutex>
 #include <QNetworkReply>
 #include <QNetworkRequest>
 #include <QSqlDatabase>
@@ -38,8 +38,11 @@
 #include "qgsauthconfig.h"
 #include "qgsauthmethod.h"
 
-// Qt5KeyChain library
-#include "keychain.h"
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <qt6keychain/keychain.h>
+#else
+#include <qt5keychain/keychain.h>
+#endif
 
 #ifndef SIP_RUN
 namespace QCA
@@ -50,8 +53,12 @@ namespace QCA
 class QgsAuthMethod;
 class QgsAuthMethodEdit;
 class QgsAuthProvider;
+class QgsAuthMethodMetadata;
+class QgsSettingsEntryBool;
 class QTimer;
-
+class QgsAuthConfigurationStorage;
+class QgsAuthConfigurationStorageDb;
+class QgsAuthConfigurationStorageRegistry;
 
 /**
  * \ingroup core
@@ -67,6 +74,9 @@ class CORE_EXPORT QgsAuthManager : public QObject
 
   public:
 
+    static const QgsSettingsEntryBool *settingsGenerateRandomPasswordForPasswordHelper SIP_SKIP;
+    static const QgsSettingsEntryBool *settingsUsingGeneratedRandomPassword SIP_SKIP;
+
     //! Message log level (mirrors that of QgsMessageLog, so it can also output there)
     enum MessageLevel
     {
@@ -83,19 +93,64 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \return TRUE on success
      * \see QgsApplication::pluginPath
      * \see QgsApplication::qgisAuthDatabaseFilePath
+     * \deprecated QGIS 3.36. Use setup() or ensureInitialized() instead.
      */
-    bool init( const QString &pluginPath = QString(),  const QString &authDatabasePath = QString() );
+    Q_DECL_DEPRECATED bool init( const QString &pluginPath = QString(),  const QString &authDatabasePath = QString() ) SIP_DEPRECATED;
+
+    /**
+     * Sets up the authentication manager configuration.
+     *
+     * This method does not initialize the authentication framework, instead that is deferred
+     * to lazy-initialize when required.
+     *
+     * \param pluginPath the plugin path
+     * \param authDatabasePath the authentication DB URI (or just the file path for SQLite)
+     * \see ensureInitialized()
+     */
+    void setup( const QString &pluginPath = QString(),  const QString &authDatabasePath = QString() );
+
+    /**
+     * Performs lazy initialization of the authentication framework, if it has
+     * not already been done.
+     *
+     * \see setup()
+     * \since QGIS 3.40
+     */
+    bool ensureInitialized() const;
 
     ~QgsAuthManager() override;
 
-    //! Sets up the application instance of the authentication database connection
-    QSqlDatabase authDatabaseConnection() const;
+    /**
+     * Returns the authentication configuration storage registry.
+     *  \since QGIS 3.40
+     */
+    QgsAuthConfigurationStorageRegistry *authConfigurationStorageRegistry() const;
 
-    //! Name of the authentication database table that stores configs
-    const QString authDatabaseConfigTable() const { return AUTH_CONFIG_TABLE; }
+    /**
+     * Sets up the application instance of the authentication database connection
+     *
+     * \deprecated QGIS 3.40. Direct access to the DB is not allowed, use QgsAuthConfigurationStorage API instead.
+     */
+    Q_DECL_DEPRECATED QSqlDatabase authDatabaseConnection() const SIP_DEPRECATED;
 
-    //! Name of the authentication database table that stores server exceptions/configs
-    const QString authDatabaseServersTable() const { return AUTH_SERVERS_TABLE; }
+    /**
+     * Name of the authentication database table that stores configs
+     * \deprecated QGIS 3.40. Direct access to the DB is not allowed, use methodConfigTableName() instead.
+     */
+    Q_DECL_DEPRECATED const QString authDatabaseConfigTable() const SIP_DEPRECATED { return AUTH_CONFIG_TABLE; }
+
+    /**
+     * Returns the database table from the first ready storage that stores authentication configs,
+     * or an empty string if none available. The table is prefixed with schema and escaped if necessary.
+     * \since QGIS 3.40
+     */
+    const QString methodConfigTableName() const;
+
+    /**
+     * Name of the authentication database table that stores server exceptions/configs
+     * \deprecated QGIS 3.40. Direct access to the DB is not allowed, use QgsAuthConfigurationStorage API instead.
+     */
+    Q_DECL_DEPRECATED const QString authDatabaseServersTable() const SIP_DEPRECATED { return AUTH_SERVERS_TABLE; }
 
 
     //! Whether QCA has the qca-ossl plugin, which a base run-time requirement
@@ -107,8 +162,42 @@ class CORE_EXPORT QgsAuthManager : public QObject
     /**
      * The standard authentication database file in ~/.qgis3/ or defined location
      * \see QgsApplication::qgisAuthDatabaseFilePath
+     * \deprecated QGIS 3.40. Use authenticationDatabaseUri() instead.
      */
-    const QString authenticationDatabasePath() const { return mAuthDbPath; }
+    Q_DECL_DEPRECATED const QString authenticationDatabasePath() const;
+
+    /**
+     * Returns the TRUE if the \a uri is a filesystem-based database (SQLite).
+     */
+    static bool isFilesystemBasedDatabase( const QString &uri );
+
+    /**
+     * Transitional proxy to the first ready storage of database type.
+     * Exposes the old SQLite local API from the new storage API.
+     * \note not available in Python bindings
+     */
+    QgsAuthConfigurationStorageDb *defaultDbStorage() const SIP_SKIP;
+
+    /**
+     * Returns the authentication database connection URI.
+     * \since QGIS 3.40
+     */
+    const QString authenticationDatabaseUri() const;
+
+    /**
+     * Returns the authentication database connection URI with the password stripped.
+     * \since QGIS 3.40
+     */
+    const QString authenticationDatabaseUriStripped() const;
+
+    /**
+     * Creates a new securely seeded random password and stores it in the
+     * system keychain as the new master password.
+     *
+     * \note Not available in Python bindings
+     * \since QGIS 3.42
+     */
+    bool createAndStoreRandomMasterPasswordInKeyChain() SIP_SKIP;
 
     /**
      * Main call to initially set or continually check master password is set
@@ -120,10 +209,10 @@ class CORE_EXPORT QgsAuthManager : public QObject
     /**
      * Overloaded call to reset master password or set it initially without user interaction
      * \note Only use this in trusted reset functions, unit tests or user/app setup scripts!
-     * \param pass Password to use
+     * \param password Password to use
      * \param verify Whether password's hash was saved in authentication database
      */
-    bool setMasterPassword( const QString &pass, bool verify = false );
+    bool setMasterPassword( const QString &password, bool verify = false );
 
     /**
      * Verify the supplied master password against any existing hash in authentication database
@@ -146,25 +235,41 @@ class CORE_EXPORT QgsAuthManager : public QObject
 
     /**
      * Check whether supplied password is the same as the one already set
-     * \param pass Password to verify
+     * \param password Password to verify
      */
-    bool masterPasswordSame( const QString &pass ) const;
+    bool masterPasswordSame( const QString &password ) const;
 
     /**
-     * Reset the master password to a new one, then re-encrypt all previous
-     * configs in a new database file, optionally backup curren database
+     * Reset the master password to a new one, then re-encrypts all previous
+     * configs with the new password.
+     *
      * \param newpass New master password to replace existing
      * \param oldpass Current master password to replace existing
-     * \param keepbackup Whether to keep the generated backup of current database
+     * \param keepbackup Whether to keep the generated backup of current database (if using file-based storage)
      * \param backuppath Where the backup is located, if kept
      */
     bool resetMasterPassword( const QString &newpass, const QString &oldpass, bool keepbackup, QString *backuppath SIP_INOUT = nullptr );
 
+    /**
+     * Reset the master password to a new one, hen re-encrypts all previous
+     * configs with the new password.
+     *
+     * The old password will automatically be retrieved from the password helper.
+     *
+     * \param newPassword New master password to replace existing
+     * \param keepBackup Whether to keep the generated backup of current database (if using file-based storage)
+     * \param backupPath Where the backup is located, if kept
+     *
+     * \note Not available in Python bindings
+     * \since QGIS 3.42
+     */
+    bool resetMasterPasswordUsingStoredPasswordHelper( const QString &newPassword, bool keepBackup, QString *backupPath = nullptr ) SIP_SKIP;
+
     /**
      * Whether there is a scheduled opitonal erase of authentication database.
      * \note not available in Python bindings
      */
-    bool scheduledAuthDatabaseErase() { return mScheduledDbErase; } SIP_SKIP
+    bool scheduledAuthDatabaseErase() SIP_SKIP { return mScheduledDbErase; }
 
     /**
      * Schedule an optional erase of authentication database, starting when mutex is lockable.
@@ -225,6 +330,13 @@ class CORE_EXPORT QgsAuthManager : public QObject
      */
     QgsAuthMethod *authMethod( const QString &authMethodKey );
 
+    /**
+     * Gets authentication method metadata via its key
+     * \param authMethodKey Authentication method key
+     * \since QGIS 3.22
+     */
+    const QgsAuthMethodMetadata *authMethodMetadata( const QString &authMethodKey ) SIP_SKIP;
+
     /**
      * Gets available authentication methods mapped to their key
      * \param dataprovider Provider key filter, returning only methods that support a particular provider
@@ -232,12 +344,17 @@ class CORE_EXPORT QgsAuthManager : public QObject
      */
     QgsAuthMethodsMap authMethodsMap( const QString &dataprovider = QString() ) SIP_SKIP;
 
+#ifdef HAVE_GUI
+    SIP_IF_FEATURE( HAVE_GUI )
+
     /**
      * Gets authentication method edit widget via its key
      * \param authMethodKey Authentication method key
      * \param parent Parent widget
      */
     QWidget *authMethodEditWidget( const QString &authMethodKey, QWidget *parent );
+    SIP_END
+#endif
 
     /**
      * Gets supported authentication method expansion(s), e.g. NetworkRequest | DataSourceURI, as flags
@@ -258,7 +375,7 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * Returns whether a string includes an authcfg ID token
      * \param txt String to check
      */
-    bool hasConfigId( const QString &txt ) const;
+    static bool hasConfigId( const QString &txt );
 
     //! Returns the regular expression for authcfg=.{7} key/value token for authentication ids
     QString configIdRegex() const { return AUTH_CFG_REGEX;}
@@ -269,9 +386,10 @@ class CORE_EXPORT QgsAuthManager : public QObject
     /**
      * Store an authentication config in the database
      * \param mconfig Associated authentication config id
+     * \param overwrite If set to TRUE, pre-existing authentication configurations will be overwritten
      * \returns Whether operation succeeded
      */
-    bool storeAuthenticationConfig( QgsAuthMethodConfig &mconfig SIP_INOUT );
+    bool storeAuthenticationConfig( QgsAuthMethodConfig &mconfig SIP_INOUT, bool overwrite = false );
 
     /**
      * Update an authentication config in the database
@@ -296,6 +414,24 @@ class CORE_EXPORT QgsAuthManager : public QObject
      */
     bool removeAuthenticationConfig( const QString &authcfg );
 
+    /**
+     * Export authentication configurations to an XML file
+     * \param filename The file path to save the XML content to
+     * \param authcfgs The list of configuration IDs to export
+     * \param password A password string to encrypt the XML content
+     * \since QGIS 3.20
+     */
+    bool exportAuthenticationConfigsToXml( const QString &filename, const QStringList &authcfgs, const QString &password = QString() );
+
+    /**
+     * Import authentication configurations from an XML file
+     * \param filename The file path from which the XML content will be read
+     * \param password A password string to decrypt the XML content
+     * \param overwrite If set to TRUE, pre-existing authentication configurations will be overwritten
+     * \since QGIS 3.20
+     */
+    bool importAuthenticationConfigsFromXml( const QString &filename, const QString &password = QString(), bool overwrite = false );
+
     /**
      * Clear all authentication configs from table in database and from provider caches
      * \returns Whether operation succeeded
@@ -370,7 +506,6 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \param defaultValue
      * \param decrypt if the value needs decrypted
      * \return QVariant( QString ) authentication setting
-     * \since QGIS 3.0
      */
     QVariant authSetting( const QString &key, const QVariant &defaultValue = QVariant(), bool decrypt = false );
 
@@ -393,7 +528,6 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \brief certIdentity get a certificate identity by \a id (sha hash)
      * \param id sha hash of the cert
      * \return the certificate
-     * \since QGIS 3.0
      */
     const QSslCertificate certIdentity( const QString &id );
 
@@ -402,7 +536,6 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \param id sha shash
      * \return a pair with the certificate and its SSL key
      * \note not available in Python bindings
-     * \since QGIS 3.0
      */
     const QPair<QSslCertificate, QSslKey> certIdentityBundle( const QString &id ) SIP_SKIP;
 
@@ -410,14 +543,12 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \brief certIdentityBundleToPem get a certificate identity bundle by \a id (sha hash) returned as PEM text
      * \param id sha hash
      * \return a list of strings
-     * \since QGIS 3.0
      */
     const QStringList certIdentityBundleToPem( const QString &id );
 
     /**
      * \brief certIdentities get certificate identities
      * \return list of certificates
-     * \since QGIS 3.0
      */
     const QList<QSslCertificate> certIdentities();
 
@@ -426,7 +557,6 @@ class CORE_EXPORT QgsAuthManager : public QObject
     /**
      * \brief certIdentityIds get list of certificate identity ids from database
      * \return list of certificate ids
-     * \since QGIS 3.0
      */
     QStringList certIdentityIds() const;
 
@@ -445,7 +575,6 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \param id sha hash
      * \param hostport string host:port
      * \return a SSL certificate custom config
-     * \since QGIS 3.0
      */
     const QgsAuthConfigSslServer sslCertCustomConfig( const QString &id, const QString &hostport );
 
@@ -453,14 +582,12 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \brief sslCertCustomConfigByHost get an SSL certificate custom config by \a hostport (host:port)
      * \param hostport host:port
      * \return a SSL certificate custom config
-     * \since QGIS 3.0
      */
     const QgsAuthConfigSslServer sslCertCustomConfigByHost( const QString &hostport );
 
     /**
      * \brief sslCertCustomConfigs get SSL certificate custom configs
      * \return list of SSL certificate custom config
-     * \since QGIS 3.0
      */
     const QList<QgsAuthConfigSslServer> sslCertCustomConfigs();
 
@@ -474,9 +601,8 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \brief ignoredSslErrorCache Get ignored SSL error cache, keyed with cert/connection's sha:host:port.
      * \return hash keyed with cert/connection's sha:host:port.
      * \note not available in Python bindings
-     * \since QGIS 3.0
      */
-    QHash<QString, QSet<QSslError::SslError> > ignoredSslErrorCache() { return mIgnoredSslErrorsCache; } SIP_SKIP
+    QHash<QString, QSet<QSslError::SslError> > ignoredSslErrorCache() SIP_SKIP { return mIgnoredSslErrorsCache; }
 
     //! Utility function to dump the cache for debug purposes
     void dumpIgnoredSslErrorsCache_();
@@ -503,7 +629,6 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \brief certAuthority get a certificate authority by \a id (sha hash)
      * \param id sha hash
      * \return a certificate
-     * \since QGIS 3.0
      */
     const QSslCertificate certAuthority( const QString &id );
 
@@ -516,28 +641,24 @@ class CORE_EXPORT QgsAuthManager : public QObject
     /**
      * \brief systemRootCAs get root system certificate authorities
      * \return list of certificate authorities
-     * \since QGIS 3.0
      */
-    const QList<QSslCertificate> systemRootCAs();
+    static const QList<QSslCertificate> systemRootCAs();
 
     /**
      * \brief extraFileCAs extra file-based certificate authorities
      * \return list of certificate authorities
-     * \since QGIS 3.0
      */
     const QList<QSslCertificate> extraFileCAs();
 
     /**
      * \brief databaseCAs get database-stored certificate authorities
      * \return list of certificate authorities
-     * \since QGIS 3.0
      */
     const QList<QSslCertificate> databaseCAs();
 
     /**
      * \brief mappedDatabaseCAs get sha1-mapped database-stored certificate authorities
      * \return sha1-mapped certificate authorities
-     * \since QGIS 3.0
      */
     const QMap<QString, QSslCertificate> mappedDatabaseCAs();
 
@@ -545,7 +666,6 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \brief caCertsCache get all CA certs mapped to their sha1 from cache.
      * \return map of sha1 <source, certificates>
      * \note not available in Python bindings
-     * \since QGIS 3.0
      */
     const QMap<QString, QPair<QgsAuthCertUtils::CaCertSource, QSslCertificate> > caCertsCache() SIP_SKIP
     {
@@ -562,7 +682,6 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \brief certTrustPolicy get whether certificate \a cert is trusted by user
      * \param cert
      * \return DefaultTrust if certificate sha not in trust table, i.e. follows default trust policy
-     * \since QGIS 3.0
      */
     QgsAuthCertUtils::CertTrustPolicy certTrustPolicy( const QSslCertificate &cert );
 
@@ -576,7 +695,6 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \brief certificateTrustPolicy get trust policy for a particular certificate \a cert
      * \param cert
      * \return DefaultTrust if certificate sha not in trust table, i.e. follows default trust policy
-     * \since QGIS 3.0
      */
     QgsAuthCertUtils::CertTrustPolicy certificateTrustPolicy( const QSslCertificate &cert );
 
@@ -589,7 +707,6 @@ class CORE_EXPORT QgsAuthManager : public QObject
     /**
      * \brief certTrustCache get cache of certificate sha1s, per trust policy
      * \return trust-policy-mapped certificate sha1s
-     * \since QGIS 3.0
      */
     const QMap<QgsAuthCertUtils::CertTrustPolicy, QStringList > certTrustCache() { return mCertTrustCache; }
 
@@ -600,14 +717,12 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \brief trustedCaCerts get list of all trusted CA certificates
      * \param includeinvalid whether invalid certs needs to be returned
      * \return list of certificates
-     * \since QGIS 3.0
      */
     const QList<QSslCertificate> trustedCaCerts( bool includeinvalid = false );
 
     /**
      * \brief untrustedCaCerts get list of untrusted certificate authorities
      * \return list of certificates
-     * \since QGIS 3.0
      */
     const QList<QSslCertificate> untrustedCaCerts( QList<QSslCertificate> trustedCAs = QList<QSslCertificate>() );
 
@@ -617,14 +732,12 @@ class CORE_EXPORT QgsAuthManager : public QObject
     /**
      * \brief trustedCaCertsCache cache of trusted certificate authorities, ready for network connections
      * \return list of certificates
-     * \since QGIS 3.0
      */
     const QList<QSslCertificate> trustedCaCertsCache() { return mTrustedCaCertsCache; }
 
     /**
      * \brief trustedCaCertsPemText get concatenated string of all trusted CA certificates
      * \return bye array with all PEM encoded trusted CAs
-     * \since QGIS 3.0
      */
     const QByteArray trustedCaCertsPemText();
 
@@ -634,7 +747,7 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * Error message getter
      * \note not available in Python bindings
      */
-    const QString passwordHelperErrorMessage() { return mPasswordHelperErrorMessage; } SIP_SKIP
+    const QString passwordHelperErrorMessage() SIP_SKIP { return mPasswordHelperErrorMessage; }
 
     /**
      * Delete master password from wallet
@@ -646,7 +759,7 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * Password helper enabled getter
      * \note Available in Python bindings since QGIS 3.8.0
      */
-    bool passwordHelperEnabled() const;
+    static bool passwordHelperEnabled();
 
     /**
      * Password helper enabled setter
@@ -658,13 +771,13 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * Password helper logging enabled getter
      * \note not available in Python bindings
      */
-    bool passwordHelperLoggingEnabled() const SIP_SKIP;
+    static bool passwordHelperLoggingEnabled() SIP_SKIP;
 
     /**
      * Password helper logging enabled setter
      * \note not available in Python bindings
      */
-    void setPasswordHelperLoggingEnabled( bool enabled ) SIP_SKIP;
+    static void setPasswordHelperLoggingEnabled( bool enabled ) SIP_SKIP;
 
     /**
      * Store the password manager into the wallet
@@ -672,12 +785,43 @@ class CORE_EXPORT QgsAuthManager : public QObject
      */
     bool passwordHelperSync();
 
-    //! The display name of the password helper (platform dependent)
+    /**
+     * Verify the password stored in the password helper.
+     *
+     * \note Not available in Python bindings
+     * \since QGIS 3.42
+     */
+    bool verifyStoredPasswordHelperPassword() SIP_SKIP;
+
+    // TODO QGIS 4.0 -- remove
+
+    /**
+     * The display name of the password helper (platform dependent).
+     *
+     * This is deprecated, use passwordHelperDisplayName() instead.
+     */
     static const QString AUTH_PASSWORD_HELPER_DISPLAY_NAME;
 
+    /**
+     * Returns a translated display name of the password helper (platform dependent).
+     *
+     * If \a titleCase is TRUE then a title case version of the string will be returned. Otherwise
+     * a mid-sentence case version will be returned.
+     *
+     * \since QGIS 3.42
+     */
+    static QString passwordHelperDisplayName( bool titleCase = false );
+
     //! The display name of the Authentication Manager
     static const QString AUTH_MAN_TAG;
 
+    /**
+     * Returns the path to the authentication database file or an empty string if the database is not SQLite.
+     *
+     * \note Not available in Python bindings
+     */
+    QString sqliteDatabasePath() const SIP_SKIP;
+
   signals:
 
     /**
@@ -698,8 +842,19 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \param tag Associated tag (title)
      * \param level Message log level
      * \see QgsMessageLog
+     * \deprecated QGIS 3.40. Use messageLog() instead.
      */
-    void messageOut( const QString &message, const QString &tag = QgsAuthManager::AUTH_MAN_TAG, QgsAuthManager::MessageLevel level = QgsAuthManager::INFO ) const;
+    Q_DECL_DEPRECATED void messageOut( const QString &message, const QString &tag = QgsAuthManager::AUTH_MAN_TAG, QgsAuthManager::MessageLevel level = QgsAuthManager::INFO ) const SIP_DEPRECATED;
+
+    /**
+     * Custom logging signal to relay to console output and QgsMessageLog
+     * \param message Message to send
+     * \param tag Associated tag (title)
+     * \param level Message log level
+     * \see QgsMessageLog
+     * \since QGIS 3.40
+     */
+    void messageLog( const QString &message, const QString &tag = QgsAuthManager::AUTH_MAN_TAG, Qgis::MessageLevel level = Qgis::MessageLevel::Info ) const;
 
     /**
      * Custom logging signal to inform the user about master password <-> password manager interactions
@@ -707,9 +862,19 @@ class CORE_EXPORT QgsAuthManager : public QObject
      * \param tag Associated tag (title)
      * \param level Message log level
      * \see QgsMessageLog
+     * \deprecated QGIS 3.40. Use passwordHelperMessageLog() instead.
      */
-    void passwordHelperMessageOut( const QString &message, const QString &tag = QgsAuthManager::AUTH_MAN_TAG, QgsAuthManager::MessageLevel level = QgsAuthManager::INFO );
+    Q_DECL_DEPRECATED void passwordHelperMessageOut( const QString &message, const QString &tag = QgsAuthManager::AUTH_MAN_TAG, QgsAuthManager::MessageLevel level = QgsAuthManager::INFO ) SIP_DEPRECATED;
 
+    /**
+     * Custom logging signal to inform the user about master password <-> password manager interactions
+     * \param message Message to send
+     * \param tag Associated tag (title)
+     * \param level Message log level
+     * \see QgsMessageLog
+     * \since QGIS 3.40
+     */
+    void passwordHelperMessageLog( const QString &message, const QString &tag = QgsAuthManager::AUTH_MAN_TAG, Qgis::MessageLevel level = Qgis::MessageLevel::Info );
 
     /**
      * Emitted when a password has been verify (or not)
@@ -731,7 +896,7 @@ class CORE_EXPORT QgsAuthManager : public QObject
     void clearCachedConfig( const QString &authcfg );
 
   private slots:
-    void writeToConsole( const QString &message, const QString &tag = QString(), QgsAuthManager::MessageLevel level = INFO );
+    void writeToConsole( const QString &message, const QString &tag = QString(), Qgis::MessageLevel level = Qgis::MessageLevel::Info );
 
     /**
      * This slot emits the authDatabaseEraseRequested signal, instead of attempting
@@ -752,7 +917,8 @@ class CORE_EXPORT QgsAuthManager : public QObject
      */
     static QgsAuthManager *instance() SIP_SKIP;
 
-#ifdef __MINGW32__
+
+#ifdef Q_OS_WIN
   public:
     explicit QgsAuthManager() SIP_SKIP;
 #else
@@ -762,6 +928,13 @@ class CORE_EXPORT QgsAuthManager : public QObject
 
   private:
 
+    /**
+     * Generates a random, securely seeded password.
+     */
+    static QString generatePassword();
+
+    bool initPrivate( const QString &pluginPath );
+
     //////////////////////////////////////////////////////////////////////////////
     // Password Helper methods
 
@@ -772,7 +945,7 @@ class CORE_EXPORT QgsAuthManager : public QObject
     void passwordHelperLog( const QString &msg ) const;
 
     //! Read Master password from the wallet
-    QString passwordHelperRead();
+    QString passwordHelperRead( bool &ok );
 
     //! Store Master password in the wallet
     bool passwordHelperWrite( const QString &password );
@@ -789,10 +962,6 @@ class CORE_EXPORT QgsAuthManager : public QObject
      */
     void passwordHelperProcessError();
 
-    bool createConfigTables();
-
-    bool createCertTables();
-
     bool masterPasswordInput();
 
     bool masterPasswordRowsInDb( int *rows ) const;
@@ -817,42 +986,29 @@ class CORE_EXPORT QgsAuthManager : public QObject
 
     bool reencryptAuthenticationIdentity( const QString &identid, const QString &prevpass, const QString &prevciv );
 
-    bool authDbOpen() const;
-
-    bool authDbQuery( QSqlQuery *query ) const;
-
-    bool authDbStartTransaction() const;
-
-    bool authDbCommit() const;
-
-    bool authDbTransactionQuery( QSqlQuery *query ) const;
-
 #ifndef QT_NO_SSL
     void insertCaCertInCache( QgsAuthCertUtils::CaCertSource source, const QList<QSslCertificate> &certs );
 #endif
 
-    const QString authDbPassTable() const { return AUTH_PASS_TABLE; }
+    QString authPasswordHelperKeyName() const;
 
-    const QString authDbSettingsTable() const { return AUTH_SETTINGS_TABLE; }
-
-    const QString authDbIdentitiesTable() const { return AUTH_IDENTITIES_TABLE; }
-
-    const QString authDbAuthoritiesTable() const { return AUTH_AUTHORITIES_TABLE; }
-
-    const QString authDbTrustTable() const { return AUTH_TRUST_TABLE; }
+    /**
+     * Returns the first ready storage with the given \a capability or NULLPTR if none available.
+     */
+    QgsAuthConfigurationStorage *firstStorageWithCapability( Qgis::AuthConfigurationStorageCapability capability ) const;
 
     static QgsAuthManager *sInstance;
     static const QString AUTH_CONFIG_TABLE;
-    static const QString AUTH_PASS_TABLE;
-    static const QString AUTH_SETTINGS_TABLE;
-    static const QString AUTH_IDENTITIES_TABLE;
     static const QString AUTH_SERVERS_TABLE;
-    static const QString AUTH_AUTHORITIES_TABLE;
-    static const QString AUTH_TRUST_TABLE;
     static const QString AUTH_CFG_REGEX;
 
+    QString mPluginPath;
+    QString mAuthDatabaseConnectionUri;
+
+    mutable bool mLazyInitResult = false;
     bool mAuthInit = false;
-    QString mAuthDbPath;
+
+    mutable std::unique_ptr<QgsAuthConfigurationStorageRegistry> mAuthConfigurationStorageRegistry;
 
     std::unique_ptr<QCA::Initializer> mQcaInitializer;
 
@@ -869,9 +1025,8 @@ class CORE_EXPORT QgsAuthManager : public QObject
     bool mScheduledDbEraseRequestEmitted = false;
     int mScheduledDbEraseRequestCount = 0;
 
-    std::unique_ptr<QMutex> mMutex;
-    std::unique_ptr<QMutex> mMasterPasswordMutex;
-
+    std::unique_ptr<QRecursiveMutex> mMutex;
+    std::unique_ptr<QRecursiveMutex> mMasterPasswordMutex;
 #ifndef QT_NO_SSL
     // mapping of sha1 digest and cert source and cert
     // appending removes duplicates
@@ -907,7 +1062,7 @@ class CORE_EXPORT QgsAuthManager : public QObject
     bool mPasswordHelperFailedInit = false;
 
     //! Master password name in the wallets
-    static const QLatin1String AUTH_PASSWORD_HELPER_KEY_NAME;
+    static const QLatin1String AUTH_PASSWORD_HELPER_KEY_NAME_BASE;
 
     //! password helper folder in the wallets
     static const QLatin1String AUTH_PASSWORD_HELPER_FOLDER_NAME;
