@@ -29,6 +29,7 @@
 #include "qgspointcloudindex.h"
 #include "qgsidentifycontext.h"
 #include "qgspointcloudrenderer.h"
+#include "qgspointcloudextentrenderer.h"
 #include "qgsmapclippingregion.h"
 #include "qgsrasterinterface.h"
 
@@ -36,16 +37,16 @@
 #include <QString>
 #include <QPainter>
 #include <QElapsedTimer>
+#include <optional>
 
 class QgsRenderContext;
 class QgsPointCloudLayer;
 class QgsPointCloudRenderer;
 class QgsPointCloudRenderContext;
+class QgsPointCloudSubIndex;
 
 #define SIP_NO_FILE
 
-///@endcond
-
 /**
  * \ingroup core
  *
@@ -65,20 +66,25 @@ class CORE_EXPORT QgsPointCloudLayerRenderer: public QgsMapLayerRenderer
     ~QgsPointCloudLayerRenderer();
 
     bool render() override;
+    Qgis::MapLayerRendererFlags flags() const override;
     bool forceRasterRender() const override;
     void setLayerRenderingTimeHint( int time ) override;
 
     QgsFeedback *feedback() const override { return mFeedback.get(); }
 
   private:
-    QVector<IndexedPointCloudNode> traverseTree( const QgsPointCloudIndex *pc, const QgsRenderContext &context, IndexedPointCloudNode n, double maxErrorPixels, double nodeErrorPixels );
-    int renderNodesSync( const QVector<IndexedPointCloudNode> &nodes, QgsPointCloudIndex *pc, QgsPointCloudRenderContext &context, QgsPointCloudRequest &request, bool &canceled );
-    int renderNodesAsync( const QVector<IndexedPointCloudNode> &nodes, QgsPointCloudIndex *pc, QgsPointCloudRenderContext &context, QgsPointCloudRequest &request, bool &canceled );
-    int renderNodesSorted( const QVector<IndexedPointCloudNode> &nodes, QgsPointCloudIndex *pc, QgsPointCloudRenderContext &context, QgsPointCloudRequest &request, bool &canceled, QgsPointCloudRenderer::DrawOrder order );
+    QVector<QgsPointCloudNodeId> traverseTree( const QgsPointCloudIndex &pc, const QgsRenderContext &context, QgsPointCloudNodeId n, double maxErrorPixels, double nodeErrorPixels );
+    int renderNodesSync( const QVector<QgsPointCloudNodeId> &nodes, QgsPointCloudIndex &pc, QgsPointCloudRenderContext &context, QgsPointCloudRequest &request, bool &canceled );
+    int renderNodesAsync( const QVector<QgsPointCloudNodeId> &nodes, QgsPointCloudIndex &pc, QgsPointCloudRenderContext &context, QgsPointCloudRequest &request, bool &canceled );
+    int renderNodesSorted( const QVector<QgsPointCloudNodeId> &nodes, QgsPointCloudIndex &pc, QgsPointCloudRenderContext &context, QgsPointCloudRequest &request, bool &canceled, Qgis::PointCloudDrawOrder order );
+    void renderTriangulatedSurface( QgsPointCloudRenderContext &context );
+    bool renderIndex( QgsPointCloudIndex &pc );
 
-    QgsPointCloudLayer *mLayer = nullptr;
+    QgsPointCloudIndex mIndex;
+    QString mLayerName;
 
     std::unique_ptr< QgsPointCloudRenderer > mRenderer;
+    std::unique_ptr< QgsPointCloudExtentRenderer > mSubIndexExtentRenderer;
 
     QgsVector3D mScale;
     QgsVector3D mOffset;
@@ -90,11 +96,19 @@ class CORE_EXPORT QgsPointCloudLayerRenderer: public QgsMapLayerRenderer
     QgsGeometry mCloudExtent;
     QList< QgsMapClippingRegion > mClippingRegions;
 
+    const QVector< QgsPointCloudSubIndex > mSubIndexes;
+    std::optional<QgsPointCloudIndex> mOverviewIndex;
+    double mAverageSubIndexWidth = 0;
+    double mAverageSubIndexHeight = 0;
+
     int mRenderTimeHint = 0;
     bool mBlockRenderUpdates = false;
     QElapsedTimer mElapsedTimer;
 
     std::unique_ptr<QgsFeedback> mFeedback = nullptr;
+
+    bool mEnableProfile = false;
+    quint64 mPreparationTime = 0;
 };
 
 #endif // QGSPOINTCLOUDLAYERRENDERER_H
