@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     MessageBarProgress.py
@@ -17,13 +15,9 @@
 ***************************************************************************
 """
 
-__author__ = 'Victor Olaya'
-__date__ = 'April 2013'
-__copyright__ = '(C) 2013, Victor Olaya'
-
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
+__author__ = "Victor Olaya"
+__date__ = "April 2013"
+__copyright__ = "(C) 2013, Victor Olaya"
 
 from qgis.PyQt.QtCore import Qt, QCoreApplication
 from qgis.PyQt.QtWidgets import QProgressBar
@@ -38,15 +32,26 @@ class MessageBarProgress(QgsProcessingFeedback):
         QgsProcessingFeedback.__init__(self)
 
         self.msg = []
-        self.progressMessageBar = \
-            iface.messageBar().createMessage(self.tr('Executing algorithm <i>{0}</i>'.format(algname if algname else '')))
+        self.progressMessageBar = iface.messageBar().createMessage(
+            self.tr("Executing algorithm <i>{}</i>".format(algname if algname else ""))
+        )
         self.progress = QProgressBar()
-        self.progressChanged.connect(self.progress.setValue)
+        self.progressChanged.connect(self.set_progress_bar_value)
         self.progress.setMaximum(100)
-        self.progress.setAlignment(Qt.AlignLeft | Qt.AlignVCenter)
+        self.progress.setAlignment(
+            Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter
+        )
         self.progressMessageBar.layout().addWidget(self.progress)
-        self.message_bar_item = iface.messageBar().pushWidget(self.progressMessageBar,
-                                                              Qgis.Info)
+        self.message_bar_item = iface.messageBar().pushWidget(
+            self.progressMessageBar, Qgis.MessageLevel.Info
+        )
+
+    def set_progress_bar_value(self, progress: float):
+        """
+        Sets the progress bar value to a rounded int of the algorithm's
+        progress
+        """
+        self.progress.setValue(int(progress))
 
     def reportError(self, msg, fatalError=False):
         self.msg.append(msg)
@@ -54,12 +59,16 @@ class MessageBarProgress(QgsProcessingFeedback):
     def close(self):
         if self.msg:
             dlg = MessageDialog()
-            dlg.setTitle(QCoreApplication.translate('MessageBarProgress', 'Problem executing algorithm'))
+            dlg.setTitle(
+                QCoreApplication.translate(
+                    "MessageBarProgress", "Problem executing algorithm"
+                )
+            )
             dlg.setMessage("<br>".join(self.msg))
-            dlg.exec_()
+            dlg.exec()
         iface.messageBar().popWidget(self.message_bar_item)
 
-    def tr(self, string, context=''):
-        if context == '':
-            context = 'MessageBarProgress'
+    def tr(self, string, context=""):
+        if context == "":
+            context = "MessageBarProgress"
         return QCoreApplication.translate(context, string)
